# -*- Perl -*-
#
# X-Rust Queue Index Definitions and Subroutines
#
# $Id: xrust_index.pl,v 1.3 1997/04/16 20:18:17 ruefenac Exp $

use strict;
use vars qw(%COLORS %REQUESTS @INDEX_INFO %INDEX_WIDGETS $query_text $query_headers $caller $IN_PRESS %FONTS);


#-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
# Text formatting strings.  The "top" and "format" lines may not
# match up here.  Thats because of fonts.  If you change fonts,
# you'll have to fiddle with these to get them lined up.
 
my $norm_top =    "RUST# User     Owner    Status    ETA         Subject";
my $norm_format = "@#### @<<<<<<< @<<<<<<< @<<<<<<<< @<<<<<<<<<< @<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<";
 
my $date_top =    "RUST# User     Owner    Age       Updated   Notified  Subject";
my $date_format = "@#### @<<<<<<< @<<<<<<< @<<<<<<<< @<<<<<<<< @<<<<<<<< @<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<";

 
my $all_date_top =    "Queue    RUST# User     Owner    Age       Updated   Notified  Subject";
my $all_date_format = "@<<<<<<< @#### @<<<<<<< @<<<<<<< @<<<<<<<< @<<<<<<<< @<<<<<<<< @<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<";

my $all_norm_top =    "Queue    RUST# User     Owner    Status    ETA          Subject";
my $all_norm_format = "@<<<<<<< @#### @<<<<<<< @<<<<<<< @<<<<<<<< @<<<<<<<<<<< @<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<";


sub create_query_index {

    my $MAINW = shift;
    my $button_width = 16;
    
    my $mainframe = $MAINW->Frame;

    my $header = $mainframe->Frame;
    $header->pack(
		  -side => 'top',
		  -fill => 'x'
		 );

    $query_headers = $header->Text(
				   -width          => 80,
				   -height         => 1,
				   -font           => $FONTS{'TEXT_FONT'},
				   -relief         => 'flat'
				  );
    push(@{$COLORS{'Query Index Header'}->{'widgets'}},$query_headers);
    
    $query_headers->pack(
			 -side => 'top',
			 -fill => 'x'
			); 
    
    $query_headers->tag('configure', 'bold',
			-font => '-*-Helvetica-Bold-R-Normal--*-180-*-*-*-*-*-*');
    

    my $noheader = $mainframe->Frame;
    $noheader->pack(
		    -side => 'bottom',
		    -fill => 'both',
		    -expand => 1
		   );

    my $buttonframe = $noheader->Frame;
    $buttonframe->pack(
		       -side => 'right',
		       -fill => 'y'
		      );

    my @button_pack = (
		       -side => 'top',
		       );

    my $button_query = $buttonframe->Button(
					    -text => 'Rescan',
					    -font => $FONTS{'BUTTON_FONT'},
					    -width => $button_width,
					    -state => 'disabled',
					    -command => \&do_query
					   );
    $button_query->pack(@button_pack);
    $INDEX_WIDGETS{'rescan'} = $button_query;

    my $button_create = $buttonframe->Button(
					     -text => 'Create',
					     -font => $FONTS{'BUTTON_FONT'},
					     -width => $button_width,
					     -state => 'disabled',
					     -command => \&create_func
					     );
    $button_create->pack(@button_pack);
    $INDEX_WIDGETS{'create'} = $button_create;

    my $button_create = $buttonframe->Button(
					     -text => 'Merge',
					     -font => $FONTS{'BUTTON_FONT'},
					     -width => $button_width,
					     -state => 'disabled'
					     );
    $button_create->pack(@button_pack);
    $INDEX_WIDGETS{'merge'} = $button_create;

    my $MW = $noheader->Frame;
    $MW->pack(
	      -side => 'left',
	      -fill => 'both',
	      -expand => 1
	     );
   
    my $vert_scroll = $MW->Scrollbar(-orient => 'vertical');
    $vert_scroll->pack(-side => 'right', -fill => 'y');
    my $horiz_scroll = $MW->Scrollbar(-orient => 'horiz');
    
    $query_text = $MW->Text(
			    -yscrollcommand =>  [$vert_scroll => 'set'],
			    -xscrollcommand =>  [$horiz_scroll => 'set'],
			    -width          => 80,
			    -height         => 10, 
			    -font           => $FONTS{'TEXT_FONT'},
			    -wrap           => 'none',
			    -setgrid        => 1
			   );
    push(@{$COLORS{'Query Index'}->{'widgets'}},$query_text);

    $vert_scroll->configure(-command => [$query_text => 'yview']);
    $horiz_scroll->configure(-command => [$query_text => 'xview']);
    $query_text->pack(
		      -fill => 'both',
		      -expand => 1
		     );
    $horiz_scroll->pack(-side => 'top', -fill => 'x');
    
    #-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
    # Create the tags to use in the text window.  This is pretty involved.
    # I took most of it from the widget demo script that comes with the
    # Tk perl module package
    
    #$query_text->bindtags([]);
    
    $query_text->tag('configure', 'sel',
		     -relief => 'flat'
		     );
    push(@{$COLORS{'Query Index'}->{'tags'}},'sel',$query_text);


    $query_text->tag('configure', 'title',
		     -font => '-*-Helvetica-Bold-R-Normal--*-180-*-*-*-*-*-*',
		    );
    
    if (0) {
	$query_text->tag('bind', 'query', 
			 '<Double-ButtonRelease-1>' => sub {
			     my $ranges = $query_text->tagRanges('hot');
			     return if ($ranges eq "");
			     my $queue_name  = $INDEX_INFO[$query_text->index('hot.first')]->{'queue_name'};
			     my $request_num = $INDEX_INFO[$query_text->index('hot.first')]->{'request_num'};
			     &create_request_window($queue_name,$request_num);
			     &display_request($queue_name,$request_num);
			 }
			);
    }
    $query_text->tag('bind', 'query', 
		     '<ButtonRelease-1>' => sub {
			 my $ranges = $query_text->tagRanges('hot');
			 return if ($ranges eq "");
			 my $queue_name  = $INDEX_INFO[$query_text->index('hot.first')]->{'queue_name'};
			 my $request_num = $INDEX_INFO[$query_text->index('hot.first')]->{'request_num'};
			 my $line_index = int($query_text->index('hot.first'));
			 my $nextline = $line_index + 1;
			 $query_text->tag('remove','selected','1.0','end');
			 $query_text->tag('add','selected',"$line_index.0","$nextline.0");
			 &display_request($queue_name,$request_num);
		     }
		    );
    
    $query_text->tag('configure', 'hot',
		     -relief => 'raised',
		     -borderwidth => 1
		    );
    push(@{$COLORS{'Query Index Selected'}->{'tags'}},'hot',$query_text);
    
    $query_text->tag('configure','selected',
		     -relief => 'raised',
		     -borderwidth => 1
		    );
    push(@{$COLORS{'Query Index Current'}->{'tags'}},'selected',$query_text);

    my $lastLine = '';
    $query_text->tag('bind', 'query', '<Enter>' => sub {
			 my $e = $query_text->XEvent;
			 my($x, $y) = ($e->x, $e->y);
			 my $lastLine = $query_text->index("\@$x,$y linestart");
			 my $nextLine = $lastLine + 1;
			 $query_text->tag('add', 'hot', $lastLine, "$nextLine.0");
		     });
    
    $query_text->tag('bind', 'query', '<Leave>' => sub {
			 $query_text->tag('remove', 'hot', '1.0', 'end');
		     });
    
    $query_text->tag('bind', 'query', '<Motion>' => sub {
			 my $e = $query_text->XEvent;
			 my($x, $y) = ($e->x, $e->y);
			 my $newLine = $query_text->index("\@$x,$y linestart");
			 #my $test = $query_text->tagRanges('hot');
			 #return if ($test eq "");
			 #my $lastLine = $query_text->index("hot.first");
			 if ((int($newLine) <= $#INDEX_INFO)) {
			     $query_text->tag('remove', 'hot', '1.0', 'end');
			     $newLine = int($newLine);
			     my $nextLine = $newLine + 1;
			     $query_text->tag('add', 'hot', "$newLine.0", "$nextLine.0");
			 }
		     });
    
    return $mainframe;
    
}

sub display_query_results {
    #
    # Abstract: Print the results of a query to the GUI
    #           window.

    my $format_type = $CONFIGURE{'format_type'};
    my $current_queue_name = $FILTER{'queue_name'};

    my $tag_title;
    my $request_num;
    my $queue_name;
    my $index;
    my $format;
    my $top;
    my $one_format;
    my $one_top;

    my @queues = %REQUESTS;

    if ($#queues > -1) {

	$query_headers->configure(-state => 'normal');
	if ($#queues == 0) {
	    if ($format_type eq "normal") {
		$query_headers->insert('end', $norm_top);
	    } else {
		$query_headers->insert('end', $date_top);
	    }
	} else {
	    if ($format_type eq "normal") {
		$query_headers->insert('end', $all_norm_top);
	    } else {
		$query_headers->insert('end', $all_date_top);
	    }
	}
	$query_headers->configure(-state => 'disabled');
	$query_text->configure(-state => 'normal');
	foreach $index (1..$#INDEX_INFO) {
	    $queue_name = $INDEX_INFO[$index]->{'queue_name'};
	    $request_num = $INDEX_INFO[$index]->{'request_num'};
	    $^A = "";
	    if ($#queues == 0) {
		if ($format_type eq "normal") {
		    formline($norm_format,
			     $request_num,
			     $REQUESTS{$queue_name}->{$request_num}->{'user'},
			     $REQUESTS{$queue_name}->{$request_num}->{'owner'},
			     $REQUESTS{$queue_name}->{$request_num}->{'status'},
			     $REQUESTS{$queue_name}->{$request_num}->{'eta'},
			     $REQUESTS{$queue_name}->{$request_num}->{'subject'}
			    );
		} else {
		    formline($date_format,
			     $request_num,
			     $REQUESTS{$queue_name}->{$request_num}->{'user'},
			     $REQUESTS{$queue_name}->{$request_num}->{'owner'},
			     $REQUESTS{$queue_name}->{$request_num}->{'age'},
			     $REQUESTS{$queue_name}->{$request_num}->{'updated'},
			     $REQUESTS{$queue_name}->{$request_num}->{'notified'},
			     $REQUESTS{$queue_name}->{$request_num}->{'subject'}
			    );
		}
	    } else {
		if ($format_type eq "normal") {
		    formline($all_norm_format,
			     $queue_name,
			     $request_num,
			     $REQUESTS{$queue_name}->{$request_num}->{'user'},
			     $REQUESTS{$queue_name}->{$request_num}->{'owner'},
			     $REQUESTS{$queue_name}->{$request_num}->{'status'},
			     $REQUESTS{$queue_name}->{$request_num}->{'eta'},
			     $REQUESTS{$queue_name}->{$request_num}->{'subject'}
			    );
		} else {
		    formline($all_date_format,
			     $queue_name,
			     $request_num,
			     $REQUESTS{$queue_name}->{$request_num}->{'user'},
			     $REQUESTS{$queue_name}->{$request_num}->{'owner'},
			     $REQUESTS{$queue_name}->{$request_num}->{'age'},
			     $REQUESTS{$queue_name}->{$request_num}->{'updated'},
			     $REQUESTS{$queue_name}->{$request_num}->{'notified'},
			     $REQUESTS{$queue_name}->{$request_num}->{'subject'}
			    );
		}
	    }
	    $query_text->insert('end',$^A, [qw(query $query_text->index('current'))]);
	    $query_text->insert('end',"\n", [qw(query $query_text->index('current'))]);
	}
	$query_text->delete('end');
	$query_text->configure(-state => 'disabled');
    
    } else {
	$query_text->insert('end',"No requests matched query");
	$query_text->configure(-state => 'disabled');
    }

}

1;


